import datetime
from typing import Tuple

import boto3
from flask import current_app

from app import app
from app.alarms import get_or_create_alarm
from app.brm.brn import BRN
from app.extensions import db
from app.models.mirrors import Proxy
from app.models.alarms import AlarmState
from app.terraform import BaseAutomation


def _cloudfront_quota() -> None:
    alarm = get_or_create_alarm(
        BRN.from_str(f"brn:{current_app.config['GLOBAL_NAMESPACE']}:0:mirror:cloudfront:quota/distributions"),
        "quota-usage"
    )
    alarm.last_updated = datetime.datetime.utcnow()
    deployed_count = len(Proxy.query.filter(
        Proxy.destroyed.is_(None)).all())
    message = f"{deployed_count} distributions deployed"
    if deployed_count > 370:
        alarm.update_state(AlarmState.CRITICAL, message)
    elif deployed_count > 320:
        alarm.update_state(AlarmState.WARNING, message)
    else:
        alarm.update_state(AlarmState.OK, message)


def _proxy_alarms() -> None:
    cloudwatch = boto3.client('cloudwatch',
                              aws_access_key_id=app.config['AWS_ACCESS_KEY'],
                              aws_secret_access_key=app.config['AWS_SECRET_KEY'],
                              region_name='us-east-2')
    dist_paginator = cloudwatch.get_paginator('describe_alarms')
    page_iterator = dist_paginator.paginate(AlarmNamePrefix="bandwidth-out-high-")
    for page in page_iterator:
        for cw_alarm in page['MetricAlarms']:
            dist_id = cw_alarm["AlarmName"][len("bandwidth-out-high-"):]
            proxy = Proxy.query.filter(Proxy.slug == dist_id).first()
            if proxy is None:
                print("Skipping unknown proxy " + dist_id)
                continue
            alarm = get_or_create_alarm(proxy.brn, "bandwidth-out-high")
            if cw_alarm['StateValue'] == "OK":
                alarm.update_state(AlarmState.OK, "CloudWatch alarm OK")
            elif cw_alarm['StateValue'] == "ALARM":
                alarm.update_state(AlarmState.CRITICAL, "CloudWatch alarm ALARM")
            else:
                alarm.update_state(AlarmState.UNKNOWN, f"CloudWatch alarm {cw_alarm['StateValue']}")


class AlarmProxyCloudfrontAutomation(BaseAutomation):
    short_name = "monitor_proxy_cloudfront"
    description = "Import alarms for AWS CloudFront proxies"

    def automate(self, full: bool = False) -> Tuple[bool, str]:
        _proxy_alarms()
        _cloudfront_quota()
        db.session.commit()
        return True, ""
